from typing import Tuple
import numpy
from dwipy import mrtrixpy
import os


def dcm2nii(
    dicom_dir: str,
    out_file: str,
    export_bvecs: bool = True,
):
    out_nifti_file = out_file + ".nii.gz"

    if export_bvecs:
        bval_file = out_file + ".bval"
        bvec_file = out_file + ".bvec"
        options = {"export_grad_fsl": [bvec_file, bval_file]}
    else:
        options = None
        bval_file = None
        bvec_file = None
    mrtrixpy.mrconvert(dicom_dir, out_nifti_file, options)

    return out_nifti_file, bval_file, bvec_file


def dcm2mif(dicom_dir: str, out_file: str, compress: bool = True) -> Tuple[str, str]:
    if compress:
        out_mif_file = out_file + ".mif.gz"
    else:
        out_mif_file = out_file + ".mif"
    grad_file = out_file + ".grads"
    if not os.path.exists(os.path.dirname(os.path.normpath(out_mif_file))):
        os.makedirs(os.path.dirname(os.path.normpath(out_mif_file)))
    print(out_mif_file)
    mrtrixpy.mrconvert(dicom_dir, out_mif_file, {"export_grad_mrtrix": grad_file})

    return out_mif_file, grad_file



import subprocess
import os
import re


def option_dict_to_list(options: dict, reverse: bool):
    options_list = []
    for i_key, i_value in options.items():
        if type(i_value) is str:
            if reverse:
                options_list.extend([str(i_value), "-" + str(i_key)])
            else:
                options_list.extend(["-" + str(i_key), str(i_value)])
        elif type(i_value) is list:
            options_list.append("-" + str(i_key))
            for i_i_value in i_value:
                options_list.append(i_i_value)
        elif type(i_value) is tuple:
            for i_i_value in i_value:
                # print(i_i_value.split(" "))
                temp = ["-" + str(i_key)]
                temp.extend(i_i_value.split(" "))
                options_list.extend(temp)
        elif i_value is None:
            options_list.append("-" + str(i_key))
    return options_list


def add_options_to_command(command, options, reverse=False):
    if options is not None:
        command.extend(option_dict_to_list(options, reverse))
        return command
    else:
        return command


def parse_mrtrix_output(mrtrix_output):
    stderr_message = mrtrix_output.stderr.decode("utf-8")
    print(stderr_message)

    warning_message = re.search("\[WARNING\](.*\n)", stderr_message)
    n_warning_messages = len(warning_message.groups()) if warning_message is not None else 0
    out_warning_messages = []
    if n_warning_messages > 0:
        for i_group_warning_message in range(n_warning_messages):
            out_warning_messages.append(warning_message.groups(i_group_warning_message)[0].strip())

    error_message = re.search("\[ERROR\](.*\n)", stderr_message)
    n_error_messages = len(error_message.groups()) if error_message is not None else 0
    out_error_messages = []
    if n_error_messages > 0:
        for i_group_error_message in range(n_error_messages):
            i_error_message = error_message.groups(i_group_error_message)[0].strip()
            # print("Error: " + i_error_message)


def mrconvert(in_file: str, out_file: str, options: dict = None):
    command = ["mrconvert"]
    command.append(in_file)
    add_options_to_command(command, options)
    command.append(out_file)
    print(command)
    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)


def mrcat(in_files: list, out_file: str):

    command = ["mrcat"]
    command.extend(in_files)

    command.append(out_file)

    # print(command)
    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)


def mrinfo(in_file: str, options: dict):
    command = ["mrinfo"]
    command.append(in_file)
    add_options_to_command(command, options)

    # command.append(out_file)

    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)


def dwi2adc(in_file: str, out_file: str, options: dict):
    command = ["dwi2adc"]
    command.append(in_file)

    add_options_to_command(command, options)

    command.append(out_file)

    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)


def mrcalc(in_file: str, out_file: str, options: dict):
    command = ["mrcalc"]
    command.append(in_file)

    add_options_to_command(command, options, reverse=True)

    command.append(out_file)

    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)


def dwishellmath(in_file: str, out_file: str, operation: str):
    command = ["dwishellmath"]
    command.append(in_file)
    command.append(operation)
    command.append(out_file)

    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)


def mrmath(in_file: str, out_file: str, operation: str, options: dict):
    command = ["mrmath"]
    command.append(in_file)
    command.append(operation)
    add_options_to_command(command, options)
    command.append(out_file)

    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)


def dwi2tensor(in_file: str, out_file: str, options: dict):
    command = ["dwi2tensor"]
    command.append(in_file)

    add_options_to_command(command, options)

    command.append(out_file)

    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)


def tensor2metric(in_file: str, options: dict):
    command = ["tensor2metric"]
    command.append(in_file)

    add_options_to_command(command, options)

    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)


def dwiextract(in_file: str, out_file: str, options: dict):
    command = ["dwiextract"]
    command.append(in_file)

    add_options_to_command(command, options)
    command.append(out_file)

    output = subprocess.run(command, capture_output=True)

    parse_mrtrix_output(output)



from typing import Tuple
import numpy as np


def load_mrtrix_grads(grad_file: str) -> np.ndarray:
    """
    Load a gradient file from the mrtrix3 format.

    Args:
        grad_file (str): File location

    Returns:
        np.ndarray: Gradients, [Nx4] matrix with N the number of entries
    """
    # Load the array with minimum of 2 dimension
    # Otherwise in case of a single line we get 1 dimensional array
    # Which is confusing for index
    grad_matrix = np.loadtxt(grad_file, delimiter=" ", comments="#", dtype=float, ndmin=2)

    # Round the bvals, they should be int
    # But in some cases, there are small floats
    grad_matrix[:, 3] = np.around(grad_matrix[:, 3])

    return grad_matrix


def get_mrtrix_bvals(grad_matrix: np.ndarray) -> np.ndarray:
    """
    Get the b-values from a gradient matrix.

    Args:
        grad_matrix (np.ndarray): Gradient matrix

    Returns:
        np.ndarray: The b-values
    """
    # Bvals are last column
    return grad_matrix[:, 3]


def load_mrtrix_bvals(grad_file: str) -> np.ndarray:
    """
    Load b-values from a gradient file.

    Args:
        grad_file (str): Path to gradient file

    Returns:
        np.ndarray: The b-values
    """
    grad_matrix = load_mrtrix_grads(grad_file)
    return get_mrtrix_bvals(grad_matrix)


def get_mrtrix_bvecs(grad_matrix: np.ndarray) -> np.ndarray:
    """
    Get the gradient vectors from a gradient matrix.

    Args:
        grad_matrix (np.ndarray): Gradient matrix

    Returns:
        np.ndarray: The gradient vectors
    """
    # Bvecs are everything up to last column
    return grad_matrix[:, 0:3]


def load_mrtrix_bvecs(grad_file: str) -> np.ndarray:
    """
    Load the gradient vectors from a gradient file

    Args:
        grad_file (str): Path to gradient file

    Returns:
        np.ndarray: The gradient vectors
    """
    grad_matrix = load_mrtrix_grads(grad_file)
    return get_mrtrix_bvecs(grad_matrix)


def get_mrtrix_bvals_and_bvecs(grad_matrix: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
    """
    Get the b-values and gradient vectors from a gradient matrix.

    Args:
        grad_matrix (np.ndarray): The gradient matrix

    Returns:
        Tuple[np.ndarray, np.ndarray]: The b-values and gradient vectors
    """
    return get_mrtrix_bvals(grad_matrix), get_mrtrix_bvecs(grad_matrix)


def write_mrtrix_grads(grad_file: str, grads: np.ndarray) -> str:
    """
    Write a gradient table to a mrtrix3 file.

    Args:
        grad_file (str): Path to the gradient file
        grads (np.ndarray): Gradient table to write

    Raises:
        ValueError: If gradient table does not have the right shape

    Returns:
        str: The path of the written gradient file
    """
    if grads.shape[1] != 4:
        raise ValueError(
            "Gradient table needs to have 4 columns. Your gradient table has {columns} instead".format(
                columns=grads.shape[1]
            )
        )

    np.savetxt(grad_file, grads, delimiter=" ", fmt=["%2.5f", "%2.5f", "%2.5f", "%d"])

    return grad_file



from enum import Enum


class GRADSTATUS(Enum):
    GRADS_NOT_FOUND = -1
    VALID_GRADS = 1
    ONE_BVAL = 2
    ONE_BVAL_SINGLE_IMAGE = 3
    ONE_BVAL_MULTI_IMAGE = 4
    BVALS_TOO_HIGH = 5
    BVALS_HAS_NAN = 6
    DTI_ONE_DIRECTION = 7


class DWITYPE(Enum):
    UNKNOWN = -1
    DWI_SINGLE_TRACE = 1
    DWI_MULTI_TRACE = 2
    DWI_MULTI_B_VECS = 3
    DTI = 4
    DWI_TRACE_NONZERO_BVAL = 5
    DWI_MULTI_B_VECS_AND_TRACE = 6


MAX_B_VALUE = 5000



import os
import shutil
import glob

import numpy as np

import dwipy.constants
import dwipy.IO.gradient_table
import dwipy.IO.misc
import dwipy.mrtrixpy
import dwipy.qa


def get_adc_from_image(image_file: str, grad_file: str, out_folder: str):

    dwi_type = dwipy.qa.get_dwi_type(grad_file)
    if dwi_type is dwipy.constants.DWITYPE.DWI_SINGLE_TRACE:
        _get_adc_from_trace_dwi(image_file, out_folder)
    elif dwi_type is dwipy.constants.DWITYPE.DWI_MULTI_B_VECS:
        _get_adc_from_dwi_with_bvecs(image_file, grad_file, out_folder)
    elif (
        dwi_type is dwipy.constants.DWITYPE.DWI_MULTI_TRACE
        or dwi_type is dwipy.constants.DWITYPE.DWI_TRACE_NONZERO_BVAL
    ):
        _get_adc_from_multi_trace_dwi(image_file, grad_file, out_folder)
    elif dwi_type is dwipy.constants.DWITYPE.DWI_MULTI_B_VECS_AND_TRACE:
        _get_adc_from_dwi_with_bvecs_and_trace(image_file, grad_file, out_folder)
    elif dwi_type is dwipy.constants.DWITYPE.DTI:
        _get_adc_from_dti(image_file, out_folder)

    return glob.glob(os.path.join(out_folder, "*_ADC.nii.gz"))


def _normalize_adc(adc_file: str):
    tmpdir = dwipy.IO.misc._get_tmp_dir()
    adc_multiplied_file = os.path.join(tmpdir, "ADC_multiplied.nii.gz")
    dwipy.mrtrixpy.mrcalc(adc_file, adc_multiplied_file, {"multiply": "1000"})

    adc_low_threshold_file = os.path.join(tmpdir, "ADC_low_threshold.nii.gz")
    dwipy.mrtrixpy.mrcalc(adc_multiplied_file, adc_low_threshold_file, {"max": "0"})

    adc_high_threshold_file = os.path.join(tmpdir, "ADC_high_threshold.nii.gz")
    dwipy.mrtrixpy.mrcalc(adc_low_threshold_file, adc_high_threshold_file, {"min": "100"})

    return adc_high_threshold_file


def _get_adc_from_trace_dwi(image_file, out_folder):
    tmpdir = dwipy.IO.misc._get_tmp_dir()

    file_name_basis = dwipy.IO.misc.get_file_base_name_without_extension(image_file)
    adc_and_b0_file = os.path.join(tmpdir, "ADC_B0.nii.gz")

    # Convert the image to a combined ADC and B0 file
    dwipy.mrtrixpy.dwi2adc(image_file, adc_and_b0_file, {})

    # First we extract the ADC file
    adc_extracted_file = os.path.join(tmpdir, "ADC_extracted.nii.gz")
    dwipy.mrtrixpy.mrconvert(
        adc_and_b0_file, adc_extracted_file, {"coord": ["3", "1"], "axes": "0,1,2"}
    )

    adc_final_file = _normalize_adc(adc_extracted_file)

    b0_extracted_file = os.path.join(tmpdir, "B0.nii.gz")
    dwipy.mrtrixpy.mrconvert(
        adc_and_b0_file, b0_extracted_file, {"coord": ["3", "0"], "axes": "0,1,2"}
    )

    shutil.copy(adc_final_file, os.path.join(out_folder, file_name_basis + "_ADC.nii.gz"))
    shutil.copy(b0_extracted_file, os.path.join(out_folder, file_name_basis + "_B0.nii.gz"))


def _get_adc_from_dwi_with_bvecs(image_file: str, grad_file: str, out_folder):
    # in this case we have a DWI with bvecs
    # This means we have multiple vectors for a certain bvalue
    # But this vectors are (approximately) only in x/y/z direction
    # Also, we can have multiple b-values that have 3 b-vectors in the x/y/z direction
    tmpdir = dwipy.IO.misc._get_tmp_dir()

    file_name_basis = dwipy.IO.misc.get_file_base_name_without_extension(image_file)

    bvals = dwipy.IO.gradient_table.load_mrtrix_bvals(grad_file)
    unique_bval, idx, bval_counts = np.unique(bvals, return_index=True, return_counts=True)
    # By default unique sorts the bvals
    # But we want them in their original order
    original_order_bvals = bvals[np.sort(idx)]
    individual_b_images = {}
    grad_scheme = {}
    for idx, (i_unique_bval, i_count) in enumerate(zip(original_order_bvals, bval_counts)):
        # First we take the mean over the b-shells
        out_file_this_b = os.path.join(tmpdir, "TRACE_B" + str(i_unique_bval) + ".mif.gz")

        if i_count > 1:
            this_b_indexes = np.squeeze(np.argwhere(bvals == i_unique_bval))
            this_b_indexes = [str(int(i_this_b_indexes)) for i_this_b_indexes in this_b_indexes]
            this_b_indexes = ",".join(this_b_indexes)
            dwipy.mrtrixpy.mrconvert(image_file, out_file_this_b, {"coord": ["3", this_b_indexes]})
            out_file_this_b_product = os.path.join(
                tmpdir, "TRACE_B_PROD_" + str(i_unique_bval) + "_COR.mif.gz"
            )
            dwipy.mrtrixpy.mrmath(
                out_file_this_b, out_file_this_b_product, "product", {"axis": "3"}
            )
            out_file_this_b_corrected = os.path.join(
                tmpdir, "TRACE_B" + str(i_unique_bval) + "_COR.mif.gz"
            )
            dwipy.mrtrixpy.mrcalc(
                out_file_this_b_product,
                out_file_this_b_corrected,
                {"pow": str(np.around(1 / i_count, decimals=5))},
            )
            out_file_this_b = out_file_this_b_corrected
        else:
            dwipy.mrtrixpy.mrconvert(
                image_file, out_file_this_b, {"coord": ["3", str(int(idx))], "axes": "0,1,2"}
            )
        individual_b_images[i_unique_bval] = out_file_this_b
        grad_scheme[i_unique_bval] = "dw_scheme 0,0,0," + str(int(i_unique_bval))

    max_b = np.amax(list(individual_b_images.keys()))
    min_b = np.amin(list(individual_b_images.keys()))
    final_trace_file = os.path.join(tmpdir, "FINAL_TRACE.mif.gz")
    dwipy.mrtrixpy.mrcat([individual_b_images[min_b], individual_b_images[max_b]], final_trace_file)

    final_trace_file_with_grads = os.path.join(tmpdir, "FINAL_TRACE_GRADS.mif.gz")
    dwipy.mrtrixpy.mrconvert(
        final_trace_file,
        final_trace_file_with_grads,
        {
            "clear_property": "dw_scheme",
            "append_property": tuple([grad_scheme[min_b], grad_scheme[max_b]]),
        },
    )
    trace_file_in_output = os.path.join(out_folder, file_name_basis + "_TRACE.mif.gz")

    shutil.copy(final_trace_file_with_grads, trace_file_in_output)

    _get_adc_from_trace_dwi(trace_file_in_output, out_folder)


def _get_adc_from_dwi_with_bvecs_and_trace(image_file: str, grad_file: str, out_folder):
    tmpdir = dwipy.IO.misc._get_tmp_dir()

    file_name_basis = dwipy.IO.misc.get_file_base_name_without_extension(image_file)
    trace_file = os.path.join(tmpdir, "TRACE.mif.gz")

    grads = dwipy.IO.gradient_table.load_mrtrix_grads(grad_file)
    # non_zero_grads_index = np.squeeze(np.argwhere(grads[:, 3] > 0 ))
    # non_zero_grad_bvecs = grads[non_zero_grads_index, 0:3]
    total_bvec = np.sum(np.power(grads[:, 0:3], 2), axis=1)
    total_bvec_is_zero_index = np.squeeze(
        np.argwhere(np.bitwise_and(total_bvec == 0, grads[:, 3] > 0))
    )

    b0_volume = np.argwhere(grads[:, 3] == 0)

    dwi_b0_file = os.path.join(tmpdir, file_name_basis + "_B0.mif.gz")
    dwi_b1000_file = os.path.join(tmpdir, file_name_basis + "_B1000.mif.gz")

    dwipy.mrtrixpy.mrconvert(
        image_file,
        dwi_b0_file,
        {"coord": ["3", str(int(np.squeeze(b0_volume[0])))], "axes": "0,1,2"},
    )

    dwipy.mrtrixpy.mrconvert(
        image_file,
        dwi_b1000_file,
        {"coord": ["3", str(int(total_bvec_is_zero_index))], "axes": "0,1,2"},
    )

    dwi_single_trace_file = os.path.join(tmpdir, file_name_basis + "_TRACE.mif.gz")

    dwipy.mrtrixpy.mrcat([dwi_b0_file, dwi_b1000_file], dwi_single_trace_file)

    _get_adc_from_trace_dwi(dwi_single_trace_file, out_folder)
    shutil.copy(dwi_single_trace_file, os.path.join(out_folder, file_name_basis + "_TRACE.mif.gz"))


def _get_adc_from_dti(image_file: str, out_folder):
    tmpdir = dwipy.IO.misc._get_tmp_dir()

    file_name_basis = dwipy.IO.misc.get_file_base_name_without_extension(image_file)
    tensor_file = os.path.join(tmpdir, "DT.mif.gz")
    adc_file = os.path.join(tmpdir, "ADC.nii.gz")

    dwipy.mrtrixpy.dwi2tensor(image_file, tensor_file, {})
    dwipy.mrtrixpy.tensor2metric(tensor_file, {"adc": adc_file})

    adc_final_file = _normalize_adc(adc_file)

    shutil.copy(adc_final_file, os.path.join(out_folder, file_name_basis + "_ADC.nii.gz"))

    b0_extracted_file = os.path.join(tmpdir, "B0.nii.gz")

    dwipy.mrtrixpy.dwiextract(image_file, b0_extracted_file, {"bzero": None})

    shutil.copy(b0_extracted_file, os.path.join(out_folder, file_name_basis + "_B0.nii.gz"))


def _get_adc_from_multi_trace_dwi(image_file: str, grad_file: str, out_folder: str):
    tmpdir = dwipy.IO.misc._get_tmp_dir()

    bvals = dwipy.IO.gradient_table.load_mrtrix_bvals(grad_file)
    # Need to find the 0 bval and hopefully the 1000 bval

    # Before we checked for b=0 and b=1000, but cannot guarantee that those are actually
    # The values there are, sometime b=50 is low and b=800 is high
    # Perhaps add a check for this.
    if 0 in bvals:
        b0_volume = np.argwhere(bvals == 0)[0]
    else:
        b0_volume = np.argmin(bvals)
    if 1000 in bvals:
        b1000_volume = np.argwhere(bvals == 1000)[0]
    else:
        b1000_volume = np.argmax(bvals)

    # If they're the same we have a problem
    if b0_volume == b1000_volume:
        return

    else:
        file_name_basis = dwipy.IO.misc.get_file_base_name_without_extension(image_file)

        dwi_b0_file = os.path.join(tmpdir, file_name_basis + "_B0.mif.gz")
        dwi_b1000_file = os.path.join(tmpdir, file_name_basis + "_B1000.mif.gz")

        dwipy.mrtrixpy.mrconvert(
            image_file,
            dwi_b0_file,
            {"coord": ["3", str(int(np.squeeze(b0_volume)))], "axes": "0,1,2"},
        )

        dwipy.mrtrixpy.mrconvert(
            image_file,
            dwi_b1000_file,
            {"coord": ["3", str(int(np.squeeze(b1000_volume)))], "axes": "0,1,2"},
        )

        dwi_single_trace_file = os.path.join(tmpdir, file_name_basis + "_TRACE.mif.gz")

        dwipy.mrtrixpy.mrcat([dwi_b0_file, dwi_b1000_file], dwi_single_trace_file)

        _get_adc_from_trace_dwi(dwi_single_trace_file, out_folder)
        shutil.copy(
            dwi_single_trace_file, os.path.join(out_folder, file_name_basis + "_TRACE.mif.gz")
        )



import os

import numpy as np

import dwipy.constants
import dwipy.IO.gradient_table


def check_grad_file_status(grad_file: str) -> dwipy.constants.GRADSTATUS:
    """
    Check the status of the gradients from a gradient file.

    Args:
        grad_file (str): Path to gradient file

    Returns:
        dwipy.constants.GRADSTATUS: The gradient status
    """
    if os.path.exists(grad_file):
        return check_grads_status(dwipy.IO.gradient_table.load_mrtrix_grads(grad_file))
    else:
        return dwipy.constants.GRADSTATUS.GRADS_NOT_FOUND


def check_grads_status(grads: np.ndarray) -> dwipy.constants.GRADSTATUS:
    bvals, bvecs = dwipy.IO.gradient_table.get_mrtrix_bvals_and_bvecs(grads)

    if any(np.isnan(bvals)):
        return dwipy.constants.GRADSTATUS.BVALS_HAS_NAN

    unique_bvals = np.unique(bvals)
    n_unique_bvals = len(unique_bvals)

    bvecs_non_zero = bvecs[bvals != 0, :]
    bvecs_total_per_direction = np.sum(np.abs(bvecs_non_zero), axis=0)

    if any(bvecs_total_per_direction == 0) and any(bvecs_total_per_direction > 0):
        return dwipy.constants.GRADSTATUS.DTI_ONE_DIRECTION
    if n_unique_bvals < 2:
        return dwipy.constants.GRADSTATUS.ONE_BVAL
    if any(unique_bvals > 3500):
        return dwipy.constants.GRADSTATUS.BVALS_TOO_HIGH
    else:
        return dwipy.constants.GRADSTATUS.VALID_GRADS


def grad_file_is_valid(grad_file: str) -> bool:
    """
    Check the validity of a gradient file.

    Args:
        grad_file (str): Path to gradient file

    Returns:
        bool: True if gradients are valid, False otherwise
    """
    return check_grad_file_status(grad_file) is dwipy.constants.GRADSTATUS.VALID_GRADS


def get_dwi_type(grad_file: str) -> dwipy.constants.DWITYPE:
    grad_matrix = dwipy.IO.gradient_table.load_mrtrix_grads(grad_file)

    bvecs = dwipy.IO.gradient_table.get_mrtrix_bvecs(grad_matrix)
    bvals = dwipy.IO.gradient_table.get_mrtrix_bvals(grad_matrix)

    bvec_total_value = np.sum(np.power(bvecs, 2), axis=1)

    if len(bvals) == 2 and np.count_nonzero(bvecs) == 0:
        return dwipy.constants.DWITYPE.DWI_SINGLE_TRACE
    elif len(bvals) > 2 and np.count_nonzero(bvecs) == 0:
        return dwipy.constants.DWITYPE.DWI_MULTI_TRACE
    elif np.count_nonzero(np.unique(bvals)) == np.count_nonzero(bvec_total_value):
        # We have bvecs but with just one corresonding bval for each
        # Not really a DTI, probably error in extraction
        return dwipy.constants.DWITYPE.DWI_TRACE_NONZERO_BVAL
    elif np.count_nonzero(bvecs) == 0:
        return dwipy.constants.DWITYPE.UNKNOWN
    else:
        # Bvec values for non-zero b values
        max_bvec = np.amax(np.abs(bvecs[bvals != 0, :]), axis=1)
        is_zero = np.isclose(bvec_total_value[bvals != 0], 0)

        if len(np.unique(bvals[bvals != 0])) * 3 == np.count_nonzero(max_bvec):
            # We can have multiple bvals that each a single x/y/z bvec
            # We do not consider this DTI
            return dwipy.constants.DWITYPE.DWI_MULTI_B_VECS
        elif np.sum(is_zero) != 0:
            # Some non-zero bvals are actually trace images
            return dwipy.constants.DWITYPE.DWI_MULTI_B_VECS_AND_TRACE
        else:
            return dwipy.constants.DWITYPE.DTI



try:
    from PIL import Image
except ImportError:
    import Image

import os
import re

from glob import glob
import shutil

import cv2
import numpy as np
import pydicom
import pytesseract
import SimpleITK as sitk

import dwipy.IO.gradient_table
import dwipy.IO.misc
import dwipy.constants
import dwipy.IO.converters
import dwipy.mrtrixpy
import dwipy.qa

# This requires the installation of tesseract 4.1.1 according to: https://medium.com/quantrium-tech/installing-tesseract-4-on-ubuntu-18-04-b6fcd0cbd78f

#############
## Functions to extract b-values if not directly found
###########
def extract_b_value_from_burned_in_annotation(DICOM_path):
    dicom_files = glob(os.path.join(DICOM_path, "*.dcm"))

    n_dcm_files_no_data = 0

    for i_dcm_file in dicom_files:
        fixed_b = False
        print(i_dcm_file)
        img_array = sitk.GetArrayFromImage(sitk.ReadImage(i_dcm_file))
        img_array = np.squeeze(img_array)
        # Assume high intensity for the text itself
        img_array[img_array != 2000] = 0
        img_array = cv2.resize(img_array, None, fx=6, fy=6, interpolation=cv2.INTER_CUBIC)

        try:
            ocr_text = pytesseract.image_to_string(
                img_array,
                config="--psm 13 -c load_system_dawg=false -c load_freq_dawg=false -c tessedit_char_whitelist==ADCadcbTSPR0123456789",
            )
        except:
            ocr_text = None

        if ocr_text is not None:
            ocr_text = ocr_text.strip()
            if "b=" in ocr_text or "5=" in ocr_text:
                if "b=" in ocr_text:
                    bval = ocr_text.split("b=")[-1]
                if "5=" in ocr_text:
                    bval = ocr_text.split("5=")[-1]
                # print(bval)
                last_zero = bval.rfind("0")
                # We cut off everything after 0 if there is any weird things left
                bval_only = bval[: last_zero + 1]
                bval_direction = bval[last_zero + 1 :]

                out_bval_direction = None

                if bval_direction == "T" or bval_direction == "7":
                    out_bval_direction = [0, 0, 0]
                elif bval_direction == "P":
                    out_bval_direction = [0, 1, 0]
                elif bval_direction == "S" or bval_direction == "5":
                    out_bval_direction = [0, 0, 1]
                elif bval_direction == "R" or bval_direction == "3":
                    out_bval_direction = [1, 0, 0]

                if bval_only == "S00":
                    bval_only = "500"
                elif bval_only == "S0":
                    bval_only = "50"
                # print(bval_only)
                i_dcm = pydicom.dcmread(i_dcm_file)
                i_dcm.add_new([0x0018, 0x9087], "DS", bval_only)
                if out_bval_direction is not None:
                    i_dcm.add_new([0x0018, 0x9089], "FD", out_bval_direction)
                pydicom.dcmwrite(i_dcm_file, i_dcm)
                fixed_b = True
            elif "abc" in ocr_text.lower() or "adc" in ocr_text.lower():
                # ADC slice, we don't want that.
                os.remove(i_dcm_file)
                fixed_b = True

        if not fixed_b:
            n_dcm_files_no_data += 1

        if n_dcm_files_no_data > 5:
            break
    return fixed_b


def extract_b_value_from_description(DICOM_path):
    dicom_files = glob(os.path.join(DICOM_path, "*.dcm"))

    fixed_b = False

    for i_dicom_file in dicom_files:
        i_dcm = pydicom.dcmread(i_dicom_file)
        diffusion_value = None
        if [0x18, 0x24] in i_dcm:
            sequence_name = i_dcm[0x18, 0x24].value
            diffusion_regex_expressions = ["ep_b(\d+)", "re_b(\d+)"]
            i_regex_expression = 0
            while diffusion_value is None and i_regex_expression < len(diffusion_regex_expressions):
                diffusion_value = re.search(
                    diffusion_regex_expressions[i_regex_expression], sequence_name
                )
                if diffusion_value is not None:
                    diffusion_value = diffusion_value.groups(0)[0].strip()
                i_regex_expression += 1
        if [0x0018, 0x1030] in i_dcm and diffusion_value is None:
            protocol_name = i_dcm[0x18, 0x1030].value
            diffusion_value = re.search("eB(\d+)", protocol_name)
            if diffusion_value is not None:
                diffusion_value = diffusion_value.groups(0)[0].strip()
            else:
                if protocol_name == "eBo":
                    diffusion_value = 0

        if [0x8, 0x8] in i_dcm and diffusion_value is None:
            image_type = i_dcm[0x8, 0x8].value

            if isinstance(image_type, pydicom.multival.MultiValue):
                image_type = "/".join(list(image_type))

            diffusion_value = re.search("DW_(\d+)", image_type)
            if diffusion_value is not None:
                diffusion_value = diffusion_value.groups(0)[0].strip()

        # if diffusion_value is not None:
        #     i_dcm.add_new([0x0018, 0x9087], "DS", diffusion_value)

        #     pydicom.dcmwrite(i_dicom_file, i_dcm)
        #     if not fixed_b:
        #         fixed_b = True
    return fixed_b


def deep_search_b_values(dicom_path: str):
    # First we try to extract them from the description
    fixed_b = extract_b_value_from_description(dicom_path)
    if not fixed_b:
        # We now try to get it from burned in annotation
        fixed_b = extract_b_value_from_burned_in_annotation(dicom_path)

    return fixed_b


#########
## Fixes for b-values that are too high (i.e. 100001000)
########


def fix_high_b_value(DICOM_path):
    # GE specific fix
    dicom_files = glob(os.path.join(DICOM_path, "*.dcm"))

    fixed_b = False

    for i_dicom_file in dicom_files:
        i_dcm = pydicom.dcmread(i_dicom_file)
        diffusion_value = None
        if [0x0043, 0x1039] in i_dcm:
            dwi_values = i_dcm[0x43, 0x1039].value
            if dwi_values[0] > 5000:
                dwi_temp_val = dwi_values[0]
                dwi_temp_val = np.mod(dwi_temp_val, 1000000)
                if dwi_temp_val < 5000 and dwi_temp_val > 0:
                    dwi_values[0] = dwi_temp_val
                    diffusion_value = dwi_temp_val
                else:
                    diffusion_value = None
        if diffusion_value is not None:
            i_dcm[0x43, 0x1039].value = dwi_values

            pydicom.dcmwrite(i_dicom_file, i_dcm)
            if not fixed_b:
                fixed_b = True
    return fixed_b


###########
### Functions to fix cases where there is only 1 b value
##########


def fix_single_b_value(dicom_path):
    # First we convert to nifti to check if there are multiple images
    tmpdir = dwipy.IO.misc._get_tmp_dir()
    nifti_file = os.path.join(tmpdir, "DWI.nii.gz")
    dwipy.mrtrixpy.mrconvert(dicom_path, nifti_file)

    image_file = None
    grad_file = None
    fixed_b = False

    if has_visual_appearance_of_more_than_one_b(nifti_file):
        tmp_file = os.path.join(tmpdir, "DWI")
        image_file, grad_file = dwipy.IO.converters.dcm2mif(dicom_path, tmp_file)
        grads = dwipy.IO.gradient_table.load_mrtrix_grads(grad_file)

        fixed_b, fixed_grads = fix_high_intensity_single_b_value(nifti_file, grads)

        formatted_grad_vals = [
            "dw_scheme " + ",".join(i_grad.astype(int).astype(str)) for i_grad in fixed_grads
        ]
        # print(formatted_grad_vals)

        fixed_image_file = os.path.join(tmpdir, "DWI_FIXED_BVALS.mif.gz")
        dwipy.mrtrixpy.mrconvert(
            image_file,
            fixed_image_file,
            {"clear_property": "dw_scheme", "append_property": tuple(formatted_grad_vals)},
        )

        fixed_grads_file = os.path.join(tmpdir, "DWI_FIXED_BVALS.grads")
        dwipy.IO.gradient_table.write_mrtrix_grads(fixed_grads_file, fixed_grads)

        image_file = fixed_image_file
        grad_file = fixed_grads_file
        fixed_b = True

    return image_file, grad_file, fixed_b


def has_visual_appearance_of_more_than_one_b(nifti_file):
    more_than_one_b = False
    scan = sitk.ReadImage(nifti_file)
    if scan.GetDimension() == 4:
        scan = sitk.GetArrayFromImage(scan)
        means = []

        for i_shell in range(scan.shape[0]):
            means.append(np.mean(scan[i_shell, :]))

        more_than_one_b = np.amax(means) >= 1.1 * np.mean(means)

    return more_than_one_b


def fix_high_intensity_single_b_value(nifti_file, grads):
    # MAybe not possible in DICOM
    # Need to figure out some way to do it directly from MIF/NIFTI and then parse back the b-values
    image = sitk.ReadImage(nifti_file)
    fixed_b = False

    if image.GetDimension() == 4 and image.GetSize()[-1] == 2:
        scan = sitk.GetArrayFromImage(image)

        scan_0_mean = np.mean(scan[0, :])
        scan_1_mean = np.mean(scan[1, :])

        if scan_0_mean > 1.1 * scan_1_mean:
            grads[0, 3] = 0
            fixed_b = True
        elif scan_1_mean > 1.1 * scan_0_mean:
            grads[1, 3] = 0
            fixed_b = True

    return fixed_b, grads


####################3
## Functions for missing/too many slices
#############


def remove_superflous_slices(dicom_path):
    fixed_b = False
    dicom_files = glob(os.path.join(dicom_path, "*.dcm"))

    slice_locations = []
    for i_dcm_file in dicom_files:
        dcm = pydicom.dcmread(i_dcm_file)
        slice_locations.append(dcm[0x20, 0x1041].value)

    unique_members, counts = np.unique(slice_locations, return_counts=True, return_index=False)

    try:
        if len(np.argwhere(counts == 1)) == 1:
            to_remove_index = unique_members[int(np.squeeze(np.argwhere(counts == 1)))]
            to_remove_index = int(np.squeeze(np.argwhere(slice_locations == to_remove_index)))
            os.remove(dicom_files[to_remove_index])
            fixed_b = True
    except:
        pass

    return fixed_b


######################
# Fix nan values
######################


def fix_nan_b_vals(dicom_path):
    fixed_b = False
    dicom_files = glob(os.path.join(dicom_path, "*.dcm"))

    bvals = []
    missing_bvals = []
    for i_dcm_file in dicom_files:
        dcm = pydicom.dcmread(i_dcm_file)
        if [0x18, 0x9087] in dcm:
            bvals.append(dcm[0x18, 0x9087].value)
        else:
            missing_bvals.append(i_dcm_file)

    if len(missing_bvals) < len(dicom_files) and np.count_nonzero(bvals) > 0:
        for i_dcm_file in missing_bvals:
            dcm = pydicom.dcmread(i_dcm_file)
            dcm.add_new([0x0018, 0x9087], "DS", 0)
            pydicom.dcmwrite(i_dcm_file, dcm)
            fixed_b = True
    return fixed_b


###############3
# Fix DTI with only one directoin
#####


def fix_dti_one_direction(dicom_path):
    # First we convert to nifti to check if there are multiple images
    tmpdir = dwipy.IO.misc._get_tmp_dir()
    tmp_file = os.path.join(tmpdir, "DWI")
    image_file, grad_file = dwipy.IO.converters.dcm2mif(dicom_path, tmp_file)
    grads = dwipy.IO.gradient_table.load_mrtrix_grads(grad_file)

    # Fix the grads
    grads[:, 0:3] = 0
    formatted_grad_vals = [
        "dw_scheme " + ",".join(i_grad.astype(int).astype(str)) for i_grad in grads
    ]

    fixed_image_file = os.path.join(tmpdir, "DWI_FIXED_BVALS.mif.gz")
    dwipy.mrtrixpy.mrconvert(
        image_file,
        fixed_image_file,
        {"clear_property": "dw_scheme", "append_property": tuple(formatted_grad_vals)},
    )

    fixed_grads_file = os.path.join(tmpdir, "DWI_FIXED_BVALS.grads")
    dwipy.IO.gradient_table.write_mrtrix_grads(fixed_grads_file, grads)

    image_file = fixed_image_file
    grad_file = fixed_grads_file
    fixed_b = True

    return image_file, grad_file, fixed_b


########
## General functions
######


def rescue_dwi(dicom_path: str, bval_status: dwipy.constants.GRADSTATUS):
    image_file = None
    grad_file = None
    conversion_successful = False
    tmp_dir = dwipy.IO.misc._get_tmp_dir()
    image_file = os.path.join(tmp_dir, "DWI")
    if bval_status is dwipy.constants.GRADSTATUS.BVALS_TOO_HIGH:
        fixed_b = fix_high_b_value(dicom_path)
        if fixed_b:
            (image_file, grad_file,) = dwipy.IO.converters.dcm2mif(
                dicom_path,
                image_file,
            )
            conversion_successful = (
                dwipy.qa.check_grad_file_status(grad_file) is dwipy.constants.GRADSTATUS.VALID_GRADS
            )
    elif bval_status is dwipy.constants.GRADSTATUS.BVALS_HAS_NAN:
        fixed_b = fix_nan_b_vals(dicom_path)
        if fixed_b:
            (image_file, grad_file,) = dwipy.IO.converters.dcm2mif(
                dicom_path,
                image_file,
            )
            conversion_successful = (
                dwipy.qa.check_grad_file_status(grad_file) is dwipy.constants.GRADSTATUS.VALID_GRADS
            )

    elif bval_status is dwipy.constants.GRADSTATUS.GRADS_NOT_FOUND:
        removed_slices = False
        if dwipy.IO.misc.get_number_of_dcm_files(dicom_path) % 2 != 0:
            # Unequal number of files, need to fix this
            removed_slices = remove_superflous_slices(dicom_path)

        fixed_b = deep_search_b_values(dicom_path)
        print(fixed_b)
        if fixed_b or removed_slices:
            image_file, grad_file = dwipy.IO.converters.dcm2mif(dicom_path, image_file)
            print(dwipy.qa.check_grad_file_status(grad_file))
            conversion_successful = (
                dwipy.qa.check_grad_file_status(grad_file) is dwipy.constants.GRADSTATUS.VALID_GRADS
            )
            print("final")
            print(conversion_successful)
            print("final")
    elif bval_status is dwipy.constants.GRADSTATUS.ONE_BVAL:
        image_file, grad_file, conversion_successful = fix_single_b_value(dicom_path)
    elif bval_status is dwipy.constants.GRADSTATUS.DTI_ONE_DIRECTION:
        image_file, grad_file, conversion_successful = fix_dti_one_direction(dicom_path)
    return image_file, grad_file, conversion_successful



import os
import tempfile
import shutil
from typing import Tuple
import glob

import dwipy.constants
import dwipy.IO.converters
import dwipy.IO.misc
import dwipy.mrtrixpy
import dwipy.qa
import dwipy.rescuer
import dwipy.IO.gradient_table
import dwipy.metrics.adc


class Pipeline:
    def __init__(self) -> None:
        self.make_adc = True

    def process(self, dwi_folder: str, out_folder: str) -> bool:
        """
        Process a DWI dicom folder.

        Convert DICOM to nifti, extracts the b-values, pre-process and calculates the ADC

        Args:
            dwi_folder (str): Folder with the DICOM files of the DWI
            out_folder (str): Folder in which to put outputs

        Returns:
            bool: Whether folder was successfully processed.
        """

        tmp_folder = tempfile.mkdtemp()
        dicom_files = glob.glob(os.path.join(dwi_folder, "*.dcm"))
        for i_file in dicom_files:
            shutil.copy(
                i_file, os.path.join(tmp_folder, os.path.basename(os.path.normpath(i_file)))
            )
        dwi_folder = tmp_folder
        out_file = os.path.join(out_folder, os.path.normpath(os.path.basename("DWI.mif.gz")))

        image_file, grad_file, conversion_successful = self.convert_dicom_to_image(
            dwi_folder,
            out_file,
        )

        if not conversion_successful:
            bval_status = dwipy.qa.check_grad_file_status(grad_file)
            fixed_image_file, fixed_grad_file, conversion_successful = dwipy.rescuer.rescue_dwi(
                dwi_folder, bval_status
            )

            if (
                fixed_image_file is not None
                and os.path.exists(fixed_image_file)
                and fixed_grad_file is not None
                and os.path.exists(fixed_grad_file)
            ):
                if os.path.exists(image_file):
                    os.remove(image_file)
                if os.path.exists(grad_file):
                    os.remove(grad_file)

                shutil.copy(fixed_image_file, image_file)
                shutil.copy(fixed_grad_file, grad_file)

                image_file = fixed_image_file
                grad_file = fixed_grad_file

        print(conversion_successful)
        bval_status = dwipy.qa.check_grad_file_status(grad_file)
        if conversion_successful:
            adc_success = dwipy.metrics.adc.get_adc_from_image(image_file, grad_file, out_folder)

        return conversion_successful and adc_success, image_file

    def convert_dicom_to_image(self, dicom_folder: str, out_file: str) -> Tuple[str, str, bool]:
        # First a naieve approach where the folder is converted using mrtrix
        # Perhaps this will already work
        image_file, grad_file = dwipy.IO.converters.dcm2mif(dicom_folder, out_file)

        grad_status = dwipy.qa.check_grad_file_status(grad_file)

        return image_file, grad_file, grad_status is dwipy.constants.GRADSTATUS.VALID_GRADS

    def process_patient_folder(self, patient_folder: str):
        output_folder = os.path.join(patient_folder, "OUTPUT")
        if os.path.exists(output_folder):
            shutil.rmtree(output_folder)

        scan_folders = os.listdir(patient_folder)
        scan_folders = [
            os.path.join(patient_folder, i_scan_folder) for i_scan_folder in scan_folders
        ]

        folder_status = []
        calculated_adc = False
        for i_folder in scan_folders:
            if os.path.exists(output_folder):
                shutil.rmtree(output_folder)
            os.makedirs(output_folder)
            calculated_adc, bval_status = self.process(i_folder, output_folder)
            folder_status.append(bval_status)
            if calculated_adc:
                break

        if not calculated_adc:
            # Perhaps we have folders that are separate b-values
            # So we need to combine them
            individual_folders = []
            if os.path.exists(output_folder):
                shutil.rmtree(output_folder)
            os.makedirs(output_folder)
            for i_folder, i_status in zip(scan_folders, folder_status):
                if (
                    i_status is dwipy.constants.GRADSTATUS.ONE_BVAL_SINGLE_IMAGE
                    or i_status is dwipy.constants.GRADSTATUS.ONE_BVAL
                ):
                    individual_folders.append(i_folder)

            individual_image_files = {}
            for i_i_individual_folder, i_individual_folder in enumerate(individual_folders):
                n_slices = len(glob.glob(os.path.join(i_individual_folder, "*")))
                img_file, _, _ = self.convert_dicom_to_image(
                    i_individual_folder,
                    os.path.join(output_folder, "DWI_B_INDIVIDUAL_" + str(i_i_individual_folder)),
                )
                if n_slices in individual_image_files:
                    individual_image_files[n_slices].append(img_file)
                else:
                    individual_image_files[n_slices] = [img_file]
            for i_individiual_image_files in individual_image_files.values():
                if len(i_individiual_image_files) >= 2:
                    full_image_file = os.path.join(output_folder, "DWI.mif.gz")
                    full_grad_file = os.path.join(output_folder, "DWI.grads")
                    dwipy.mrtrixpy.mrcat(i_individiual_image_files, full_image_file)
                    dwipy.mrtrixpy.mrinfo(full_image_file, {"export_grad_mrtrix": full_grad_file})

                    if os.path.exists(full_image_file) and os.path.exists(full_grad_file):
                        calculated_adc = dwipy.metrics.adc.get_adc_from_image(
                            full_image_file, full_grad_file, output_folder
                        )
                    if calculated_adc:
                        break
        return calculated_adc



import numpy as np
import dwipy.constants
import os


class GradientTable:
    def __init__(self, gradient_matrix: np.ndarray) -> None:
        # Round the bvals, they should be int
        # But in some cases, there are small floats
        gradient_matrix[:, 3] = np.around(gradient_matrix[:, 3])
        self._gradients = gradient_matrix

    @classmethod
    def from_matrix(cls, gradient_table: np.ndarray):
        if gradient_table.shape[1] != 4:
            raise ValueError("Invalid gradient table format")

        return cls(gradient_table)

    @classmethod
    def from_file(cls, gradient_file: str):
        """
        Load a gradient file from the mrtrix3 format.

        Args:
            gradient_file (str): File location

        Returns:
            np.ndarray: Gradients, [Nx4] matrix with N the number of entries
        """
        # Load the array with minimum of 2 dimension
        # Otherwise in case of a single line we get 1 dimensional array
        # Which is confusing for index
        if os.path.exists(gradient_file):
            grad_matrix = np.loadtxt(
                gradient_file, delimiter=" ", comments="#", dtype=float, ndmin=2
            )
        else:
            raise IOError("Gradient file {grad_file} not found!".format(grad_file=gradient_file))

        return cls(grad_matrix)

    @property
    def gradients(self) -> np.ndarray:
        """
        Get the full gradient table.

        Returns:
            np.ndarray: The gradient table
        """
        return self._gradients

    @property
    def n_entries(self) -> int:
        """
        Get number of entries in gradient table.

        Returns:
            int: Number of entries
        """
        return self._gradients.shape[0]

    @property
    def n_unique_bvals(self) -> int:
        return self.unique_bvals.shape[0]

    @property
    def bvals(self) -> np.ndarray:
        """
        Get the b-values from the gradient matrix.

        Returns:
            np.ndarray: The b-values
        """
        # Bvals are last column
        return self._gradients[:, 3]

    @property
    def unique_bvals(self) -> np.ndarray:
        return np.unique(self.bvals)

    @bvals.setter
    def bvals(self, bvals_value: np.ndarray) -> None:
        if bvals_value.shape[0] != self.n_entries:
            raise ValueError(
                "Number of b-values does not match number of entries. Currently gradient table has {n_entries} entries, you provided {bval_length} entries.".format(
                    n_entries=self.n_entries, bval_length=bvals_value.shape[0]
                )
            )
        # Round the bvals to make sure they are ints
        bvals_value = np.around(bvals_value)
        self._gradients[:, 3] = bvals_value

    @property
    def bvecs(self) -> np.ndarray:
        """
        Get the gradient vectors from the gradient matrix.

        Returns:
            np.ndarray: The gradient vectors
        """
        # Bvecs are everything up to last column
        return self._gradients[:, 0:3]

    @bvecs.setter
    def bvecs(self, bvecs_values: np.ndarray) -> None:
        if bvecs_values.shape[0] != self.n_entries:
            raise ValueError(
                "Number of b-values does not match number of entries. Currently gradient table has {n_entries} entries, you provided {bvec_length} entries.".format(
                    n_entries=self.n_entries, bvec_length=bvecs_values.shape[0]
                )
            )

        self._gradients[:, 0:3] = bvecs_values

    @property
    def bvecs_non_zero_bvals(self) -> np.ndarray:
        return self.bvecs[self.bvals != 0, 0:3]

    def write_to_file(self, gradient_file: str) -> None:
        """
        Write the gradient table to a mrtrix3 file.

        Args:
            gradient_file (str): Path to the gradient file
        """
        np.savetxt(
            gradient_file, self._gradients, delimiter=" ", fmt=["%2.5f", "%2.5f", "%2.5f", "%d"]
        )

    @property
    def bvalue_status(self) -> dwipy.constants.GRADSTATUS:
        if any(np.isnan(self.bvals)):
            return dwipy.constants.GRADSTATUS.BVALS_HAS_NAN
        if self.n_unique_bvals < 2:
            return dwipy.constants.GRADSTATUS.ONE_BVAL
        if any(self.unique_bvals > dwipy.constants.MAX_B_VALUE):
            return dwipy.constants.GRADSTATUS.BVALS_TOO_HIGH

        total_bvec_per_direction = np.sum(np.abs(self.bvecs_non_zero_bvals), axis=0)

        if any(total_bvec_per_direction == 0) and any(total_bvec_per_direction > 0):
            return dwipy.constants.GRADSTATUS.DTI_ONE_DIRECTION

        return dwipy.constants.GRADSTATUS.VALID_GRADS

    def is_valid(self) -> bool:
        return self.bvalue_status is dwipy.constants.GRADSTATUS.VALID_GRADS

    @property
    def dwi_type(self) -> dwipy.constants.DWITYPE:
        if not self.is_valid():
            raise ValueError("The gradient table is not a valid DWI table, cannot determine type")

        bvec_total_value = np.sum(np.power(self.bvecs, 2), axis=1)
        if self.n_unique_bvals == 2 and np.count_nonzero(self.bvecs) == 0:
            # WE only have two entries, and all b-vectors are zero
            # So it is a trace
            return dwipy.constants.DWITYPE.DWI_SINGLE_TRACE
        elif self.n_unique_bvals > 2 and np.count_nonzero(self.bvecs) == 0:
            # We have mutiple b-values, but all are still trace
            return dwipy.constants.DWITYPE.DWI_MULTI_TRACE
        # Not sure what thise one represents
        # elif np.count_nonzero(np.unique(bvals)) == np.count_nonzero(bvec_total_value):
        #     # We have bvecs but with just one corresonding bval for each
        #     # Not really a DTI, probably error in extraction
        #     return dwipy.constants.DWITYPE.DWI_TRACE_NONZERO_BVAL
        # elif np.count_nonzero(bvecs) == 0:
        #     return dwipy.constants.DWITYPE.UNKNOWN
        else:
            # Bvec values for non-zero b values
            max_bvec = np.amax(np.abs(self.bvecs[self.bvals != 0, :]), axis=1)
            is_zero = np.isclose(bvec_total_value[self.bvals != 0], 0)

            if len(np.unique(self.bvals[self.bvals != 0])) * 3 == np.count_nonzero(max_bvec):
                # We can have multiple bvals that each a single x/y/z bvec
                # We do not consider this DTI
                return dwipy.constants.DWITYPE.DWI_MULTI_B_VECS
            elif np.sum(is_zero) != 0:
                # Some non-zero bvals are actually trace images
                return dwipy.constants.DWITYPE.DWI_MULTI_B_VECS_AND_TRACE
            else:
                return dwipy.constants.DWITYPE.DTI



import os
import tempfile
import glob


def get_file_base_name_without_extension(file_path):
    file_path = os.path.normpath(file_path)
    file_name = os.path.basename(file_path)

    return file_name.split(".")[0]


def _get_tmp_dir():
    return tempfile.mkdtemp()


def get_number_of_dcm_files(dicom_folder):
    return len(glob.glob(os.path.join(dicom_folder, "*.dcm")))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dwipy', 'dwipy.IO', 'dwipy.metrics', 'dwipy.mrtrix']

package_data = \
{'': ['*']}

install_requires = \
['SimpleITK>=2.0.2,<3.0.0',
 'numpy>=1.19.4,<2.0.0',
 'opencv-python>=4.4.0,<5.0.0',
 'pydicom>=2.1.1,<3.0.0',
 'pytesseract>=0.3.6,<0.4.0']

setup_kwargs = {
    'name': 'dwipy',
    'version': '0.2.0',
    'description': 'DWI anaylysis pipeline',
    'long_description': '# dwipy\n\n[![Build Status](https://github.com/svdvoort/dwipy/workflows/test/badge.svg?branch=master&event=push)](https://github.com/svdvoort/dwipy/actions?query=workflow%3Atest)\n[![Python Version](https://img.shields.io/pypi/pyversions/dwipy.svg)](https://pypi.org/project/dwipy/)\n[![wemake-python-styleguide](https://img.shields.io/badge/style-wemake-000000.svg)](https://github.com/wemake-services/wemake-python-styleguide)\n\nDWI analysis pipeline\n\nMade for  the initial analysis of diffusion data, especially for extracting the b-values and b-vectors from DICOM data, to allow for further processing.\n\n## Installation\n\nThis package requires the installation of Tesseract 4.1.1: https://medium.com/quantrium-tech/installing-tesseract-4-on-ubuntu-18-04-b6fcd0cbd78f\nAnd mtrix: https://mrtrix.readthedocs.io/en/latest/\n\n```bash\npip install dwipy\n```\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/svdvoort/dwipy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.0,<3.9',
}


setup(**setup_kwargs)
